@@ -22,7 +22,7 @@ SM_UNUSED(static char copyright[]) =
 #endif /* ! lint */
 
 #ifndef lint
-SM_UNUSED(static char id[]) = "@(#)$Id: editmap.c,v 1.26 2013/11/22 20:51:26 ca Exp $";
+SM_UNUSED(static char id[]) = "@(#)$Id: editmap.c,v 1.26 2013-11-22 20:51:26 ca Exp $";
 #endif /* ! lint */
 
 
@@ -76,7 +76,6 @@ main(argc, argv)
 {
 	char *progname;
 	char *cfile;
-	bool verbose = false;
 	bool query = false;
 	bool update = false;
 	bool remove = false;
@@ -131,7 +130,7 @@ main(argc, argv)
 	(void) sm_strlcpy(user_info.smdbu_name, RunAsUserName,
 			  SMDB_MAX_USER_NAME_LEN);
 
-#define OPTIONS		"C:fquxvN"
+#define OPTIONS		"C:fquxN"
 	while ((opt = getopt(argc, argv, OPTIONS)) != -1)
 	{
 		switch (opt)
@@ -159,10 +158,6 @@ main(argc, argv)
 			nops++;
 			break;
 
-		  case 'v':
-			verbose = true;
-			break;
-
 		  case 'N':
 			inclnull = true;
 			break;
